// @ts-check

/**
  Imports.

  @typedef {import('./types').Session} Session
  @typedef {import('node:events').EventEmitter} EventEmitter

  @typedef {typeof import('./types').WebSocket} WebSocket
*/

const _kitten = /** @type {Object} */ (globalThis.kitten)

export const version = /** @type {{
  "date": Date,
  "versionStamp": number,
  "gitHash": string,
  "apiVersion": number,
  "nodeVersion": string,
  "exactVersion": string,
  "birthday": string,
  "starSign": string,
  "Component": () => Function,
  "html": () => string,
  "printToConsole": () => void
}} */ (_kitten.version)

// db is the custom database. If you want type safety for it,
// create a database app module in your project and declare
// the types there.
// See: https://codeberg.org/kitten/app#database-app-module
export const db = /** @type {Object} */ (_kitten.db)

export const _db = /** @type {{
  sessions: Object.<string, Session>,
  uploads: Object.<string, Upload>
}} */ (_kitten._db)

/**
  The Upload class is copied verbatim from Kitten. This does
  create redundancy and the possibility of the two clases diverging
  in the future if we’re not careful but it seems to be the easiest
  way of getting things working at the moment. We need access to the
  Upload class itself if we’re going to be storing references to
  uploads in our own databases (outside of the internal _db database)
  e.g., in the default db database as, otherwise, the class information
  will be lost the next time the database is opened if Upload isn’t
  added to the list of classes in our database app module.

  * * *

  We convert the file upload metadata returned by Busboy
  to a leaner format that’s more ergonomic when authoring so
  we do a little bit of gymlastics with types so our intent
  is clear and the type checker is happy.

  @typedef {{
    uuid: string,
    filename: string,
    file: string,
    mimetype: string,
    field: string,
    encoding: string,
    truncated: boolean,
    done: boolean
  }} BusboyFileUploadMetadata
*/

export class Upload {
  /**
    @param {BusboyFileUploadMetadata} metadata
  */
  constructor (metadata) {
    // Fields with altered names (easier to remember and use; more consistent).
    this.id = metadata.uuid
    this.fileName = metadata.filename
    this.filePath = metadata.file

    // Fields with same names as Busboy metadata.
    this.mimetype = metadata.mimetype
    this.field = metadata.field
    this.encoding = metadata.encoding
    this.truncated = metadata.truncated
    this.done = metadata.done
  }

  /**
    Returns absolute resource path for accessing this upload via a GET request.
  */
  get resourcePath () {
    return `/uploads/${this.id}/`
  }

  /**
    Returns the absolute download path (content-disposition: attachment).
  */
  get downloadPath () {
    return `${this.resourcePath}download`
  }

  /**
    Deletes this uploaded file from the file system and also from the internal database. 
  */
  async delete () {
    // Carry out the delete at the collection level. Ideally, we
    // should’t know anything about the collection level but I
    // think we can get away with this little violation of encapsulation
    // as the alterntive would require unnecessary overhead to either
    // inject the collection reference to every upload or implement some
    // sort of event system, etc. The important thing is that we are not
    // redundantly implementing the delete functionality at both the collection
    // and the individual item level.
    globalThis.kitten._db.uploads.delete(this.id)
  }
}

export const uploads = /** @type {{
  get(id:string):Upload
  all():Upload[]
  allIds():string[]
  delete(id:string):Promise<void>
  } & Record<string, Upload>}
*/ (_kitten._db.uploads)

export const domain = /** @type {string} */ (_kitten.domain)
export const port = /** @type {number} */ (_kitten.port)

export const databaseDirectory = /** @type {string} */ (_kitten.databaseDirectory)
export const projectIdentifier = /** @type {string} */ (_kitten.projectIdentifier)

export const events = /** @type {EventEmitter} */ (_kitten.events)

export const html = /** @type {taggedTemplate} */ (_kitten.html)
export const markdown = /** @type {taggedTemplate} */ (_kitten.markdown)
export const css = /** @type {taggedTemplate} */ (_kitten.css)
export const js = /** @type {taggedTemplate} */ (_kitten.js)

export const WebSocket = /** @type {WebSocket} */ (_kitten.WebSocket)

//////////////////////////////////////////////////////////////////////////////////////////

// Kept for backwards compatibility with existing projects.

export const HTMX = /** @type {string} */ (_kitten.libraries.htmx)
export const HTMX_WEBSOCKET = /** @type {string} */ (_kitten.libraries.htmxWebSocket)
export const ALPINEJS = /** @type {string} */ (_kitten.libraries.alpineJs)
export const WATER = /** @type {string} */ (_kitten.libraries.water)

export const safelyAddHtml = /** @type {sanitisationFunction} */ (_kitten.safelyAddHtml)
export const sanitise = /** @type {sanitisationFunction} */ (_kitten.sanitise)

export const PAGE = {
  html: 'HTML',
  head: 'HEAD',
  startOfBody: 'START_OF_BODY',
  beforeLibraries: 'BEFORE_LIBRARIES',
  afterLibraries: 'AFTER_LIBRARIES',
  endOfBody: 'END_OF_BODY'
}

//////////////////////////////////////////////////////////////////////////////////////////

export const libraries = {
  htmx: HTMX,
  htmxWebSocket: HTMX_WEBSOCKET,
  alpineJs: ALPINEJS,
  water: WATER
}

export const page = PAGE

/**
  @callback taggedTemplate
  @param {TemplateStringsArray} strings
  @param {...any} properties
  @returns {string|Array<string>}
*/

/**
  @callback sanitisationFunction
  @param {string} untrustedContent
  @param {Array<string>=} allowedTags
  @param {boolean=} contact
  @returns {string}
*/

// New kitten namespace.

export const kitten = {
  version,

  db,
  _db,
  uploads,

  domain,
  port,

  databaseDirectory,
  projectIdentifier,
  
  events,

  html,
  css,
  js,
  markdown,

  libraries,
  page,

  safelyAddHtml,
  sanitise,

  WebSocket
}

export default kitten
